==> provider_software_3.7.4.tar.gz: provider_software_3.7.4/lib/biocase/configtool/cmfupgrade.py <==
#
# -*- coding: utf-8 -*-
'''
$RCSfile: cmfupgrade.py,v $
$Revision: 1416 $
$Author: j.holetschek $
$Date: 2015-01-15 14:12:48 +0100 (Do, 15. Jan 2015) $
'''

import sys, os, string
from datetime                   import datetime
from biocase.wrapper.cmf_base   import CMFClass
from biocase.wrapper.cmf_element import CMFMappingClass
import biocase.configuration

cfg = biocase.configuration.Cfg()
import logging
log = logging.getLogger("webapp.cfgtool")

class cmfUpgrade(object):
    def __init__(self):
        self.cmfUpgradeDir = os.path.join( cfg.rawCmfTemplateLocator, 'upgrades')
        # read index dictionary for upgrade files
        class Index( object ):
            pass
        indexObj = Index()
        execfile( os.path.join(self.cmfUpgradeDir,'index.py'), globals(), indexObj.__dict__ )
        # Check all upgrade tuples and keep only those for which both target raw cmf and upgrade file exist
        for source_ns, l in indexObj.index.items():
            l_new = []
            for (targetNamespace, targetRawCMFile, upgradeFile) in l:
                # Only copy to new list if both target raw cmf and upgrade file exist
                if os.path.exists(os.path.join(cfg.rawCmfTemplateLocator, targetRawCMFile)) and os.path.exists(os.path.join(cfg.rawCmfTemplateLocator, 'upgrades', upgradeFile)):
                    l_new.append((targetNamespace, targetRawCMFile, upgradeFile))
                else:
                    log.debug("Upgrade path %s --> %s removed, because either target raw CMF file %s or upgrade file %s doesn't exist." % (source_ns, targetNamespace, targetRawCMFile, upgradeFile))
            indexObj.index[source_ns] = l_new
        self.indexDict     = indexObj.index
        log.debug("Full available upgrade ditcionary:")
        log.debug(unicode(self.indexDict))
        
        
    def listPossibleUpgrades(self, sourceCmfNS):
        '''Return a list of raw cmf filenames without suffix suitable for upgrading a given cmf.'''
        if not self.indexDict.has_key(sourceCmfNS):
            return []
        return [t[1][:-4] for t in self.indexDict[sourceCmfNS]]
            
        
    def listPossibleUpgradeTuples(self, sourceCmfNS):
        '''Return a dict (key=namespace) of tuples (rawCmfFileName, upgradeFileName) 
        for all available upgrades of a given cmf namespace.'''
        if self.indexDict.has_key(sourceCmfNS):
            log.debug("List possible upgrades for ns=%s: %s" % (sourceCmfNS, unicode([t[0] for t in self.indexDict[sourceCmfNS]])))
            return self.indexDict[sourceCmfNS]
        else:
            log.debug("No possible upgrades for ns=%s"%sourceCmfNS)
            return {}

    def getUpgradedCmfObj(self, cmfFileName, targetSchema):
        '''Return a cmf object based on the upgraded namespace and the appropiate upgrade file.
        All existing mappings that couldnt be transfered are returned as a list in the second parameter.'''
        if os.path.isfile(cmfFileName):
            # read existing CMF
            oldCmf = CMFClass()
            oldCmf.loadCMFdata( cmfFileName)
            if self.indexDict.has_key(oldCmf.getRootNamespace()) or self.indexDict.has_key(oldCmf.getRootNamespace().lower()):
                # upgradeTuple
                upgradeTupleList = [(targetCmf, upgradeTxt) for targetNS, targetCmf, upgradeTxt in self.indexDict[oldCmf.getRootNamespace()] if targetCmf == targetSchema]
                if len(upgradeTupleList) > 0:
                    upgradeTuple = upgradeTupleList[0]
                else:
                    return None, [], (None,None)
            else:
                return None, [], (None,None)
        else:
            return None, [], (None,None)
        
        log.debug("UPGRADING CMF %s to Schema %s ..." % (oldCmf.getRootNamespace(), targetCmf))
        # new target cmf object
        newCmf = CMFClass()
        rawCmfPath = os.path.join( cfg.rawCmfTemplateLocator, upgradeTuple[0] )
        newCmf.loadCMFdata( rawCmfPath )
        # transfer non mapping properties
        newCmf.name           = oldCmf.name
        newCmf.generated_by   = "Configtool lib $Revision: 1416 $"
        newCmf.generated_when = str(datetime.now())
        newCmf.omitNS         = oldCmf.omitNS
        newCmf.tableTreeObj.rootTableAlias = oldCmf.tableTreeObj.rootTableAlias
        newCmf.tableTreeObj.staticTables = oldCmf.tableTreeObj.staticTables
        # transfer mappings
        # read upgrade file...
        upgradeFilePath = os.path.join( self.cmfUpgradeDir, upgradeTuple[1] )
        upgradeDict = {}
        literalsDict = {}   # key=newCmfPath, value=literal
        for line in file(upgradeFilePath).readlines():
            # Ignore lines that do not contain two XPaths separated by TAB
            # (i.e. elements that have no targetPath)
            if chr(9) in line:
                sourcePath, targetPath = line.split(chr(9))
                sourcePath = sourcePath.strip()
                targetPath = targetPath.strip()
                if len(sourcePath) > 2 and sourcePath.startswith("'") and sourcePath.endswith("'"):
                    # its a literal in simple quotes!
                    literalsDict[targetPath] = sourcePath[1:-1]
                elif upgradeDict.has_key(sourcePath):
                    upgradeDict[sourcePath].append(targetPath)
                else:
                    upgradeDict[sourcePath] = [targetPath]
        # get dict of mappings with key=concept path, value=list of mapping objects
        mappings = oldCmf.removeMappings()
        newMappings = {}
        missingConcepts = {}
        for path, map in mappings.items():
            if upgradeDict.has_key(path):
                log.debug("Move mappings from %s to %s" %(path, unicode(upgradeDict[path]) ))
                for targetPath in upgradeDict[path]:
                    if not newMappings.has_key(targetPath):
                        newMappings[targetPath] = []
                    newMappings[targetPath] += map[:]
            else:
                missingConcepts[path] = map
        # update newCmf
        newCmf.setMappings( newMappings )
        # add literals
        for path, literal in literalsDict.items():
            mapObj = CMFMappingClass(items=[literal])
            newCmf.addMappingToConcept(path=path, mapObj=mapObj)
        # log missing concepts
        log.debug("The following missing concepts couldnt be included in the upgraded CMF:")
        for path, map in missingConcepts.items():
            log.debug("Concept %s : %s" % (path, unicode(map) ))
        # return cmf and missing concepts mapping dict
        return newCmf, missingConcepts, upgradeTuple

==> provider_software_3.8.4.tar.gz: provider_software_3.8.4/lib/biocase/configtool/cmfupgrade.py <==
#
# -*- coding: utf-8 -*-
'''
$RCSfile: cmfupgrade.py,v $
$Revision: 1627 $
$Author: j.holetschek $
$Date: 2021-05-25 15:06:16 +0200 (Di, 25. Mai 2021) $
'''

import sys, os, string
from datetime                   import datetime
from biocase.wrapper.cmf_base   import CMFClass
from biocase.wrapper.cmf_element import CMFMappingClass
import biocase.configuration

cfg = biocase.configuration.Cfg()
import logging
log = logging.getLogger("webapp.cfgtool")

class cmfUpgrade(object):
    def __init__(self):
        self.cmfUpgradeDir = os.path.join( cfg.rawCmfTemplateLocator, 'upgrades')
        # read index dictionary for upgrade files
        class Index( object ):
            pass
        indexObj = Index()
        exec(compile(open( os.path.join(self.cmfUpgradeDir,'index.py'), "rb" ).read(), os.path.join(self.cmfUpgradeDir,'index.py'), 'exec'), globals(), indexObj.__dict__)
        # Check all upgrade tuples and keep only those for which both target raw cmf and upgrade file exist
        for source_ns, l in list(indexObj.index.items()):
            l_new = []
            for (targetNamespace, targetRawCMFile, upgradeFile) in l:
                # Only copy to new list if both target raw cmf and upgrade file exist
                if os.path.exists(os.path.join(cfg.rawCmfTemplateLocator, targetRawCMFile)) and os.path.exists(os.path.join(cfg.rawCmfTemplateLocator, 'upgrades', upgradeFile)):
                    l_new.append((targetNamespace, targetRawCMFile, upgradeFile))
                else:
                    log.debug("Upgrade path %s --> %s removed, because either target raw CMF file %s or upgrade file %s doesn't exist." % (source_ns, targetNamespace, targetRawCMFile, upgradeFile))
            indexObj.index[source_ns] = l_new
        self.indexDict     = indexObj.index
        log.debug("Full available upgrade ditcionary:")
        log.debug(str(self.indexDict))
        
        
    def listPossibleUpgrades(self, sourceCmfNS):
        '''Return a list of raw cmf filenames without suffix suitable for upgrading a given cmf.'''
        if sourceCmfNS not in self.indexDict:
            return []
        return [t[1][:-4] for t in self.indexDict[sourceCmfNS]]
            
        
    def listPossibleUpgradeTuples(self, sourceCmfNS):
        '''Return a dict (key=namespace) of tuples (rawCmfFileName, upgradeFileName) 
        for all available upgrades of a given cmf namespace.'''
        if sourceCmfNS in self.indexDict:
            log.debug("List possible upgrades for ns=%s: %s" % (sourceCmfNS, str([t[0] for t in self.indexDict[sourceCmfNS]])))
            return self.indexDict[sourceCmfNS]
        else:
            log.debug("No possible upgrades for ns=%s"%sourceCmfNS)
            return {}

    def getUpgradedCmfObj(self, cmfFileName, targetSchema):
        '''Return a cmf object based on the upgraded namespace and the appropiate upgrade file.
        All existing mappings that couldnt be transfered are returned as a list in the second parameter.'''
        if os.path.isfile(cmfFileName):
            # read existing CMF
            oldCmf = CMFClass()
            oldCmf.loadCMFdata( cmfFileName)
            if oldCmf.getRootNamespace() in self.indexDict or oldCmf.getRootNamespace().lower() in self.indexDict:
                # upgradeTuple
                upgradeTupleList = [(targetCmf, upgradeTxt) for targetNS, targetCmf, upgradeTxt in self.indexDict[oldCmf.getRootNamespace()] if targetCmf == targetSchema]
                if len(upgradeTupleList) > 0:
                    upgradeTuple = upgradeTupleList[0]
                else:
                    return None, [], (None,None)
            else:
                return None, [], (None,None)
        else:
            return None, [], (None,None)
        
        log.debug("UPGRADING CMF %s to Schema %s ..." % (oldCmf.getRootNamespace(), targetCmf))
        # new target cmf object
        newCmf = CMFClass()
        rawCmfPath = os.path.join( cfg.rawCmfTemplateLocator, upgradeTuple[0] )
        newCmf.loadCMFdata( rawCmfPath )
        # transfer non mapping properties
        newCmf.name           = oldCmf.name
        newCmf.generated_by   = "Configtool lib $Revision: 1627 $"
        newCmf.generated_when = str(datetime.now())
        newCmf.omitNS         = oldCmf.omitNS
        newCmf.tableTreeObj.rootTableAlias = oldCmf.tableTreeObj.rootTableAlias
        newCmf.tableTreeObj.staticTables = oldCmf.tableTreeObj.staticTables
        # transfer mappings
        # read upgrade file...
        upgradeFilePath = os.path.join( self.cmfUpgradeDir, upgradeTuple[1] )
        upgradeDict = {}
        literalsDict = {}   # key=newCmfPath, value=literal
        for line in open(upgradeFilePath).readlines():
            # Ignore lines that do not contain two XPaths separated by TAB
            # (i.e. elements that have no targetPath)
            if chr(9) in line:
                sourcePath, targetPath = line.split(chr(9))
                sourcePath = sourcePath.strip()
                targetPath = targetPath.strip()
                if len(sourcePath) > 2 and sourcePath.startswith("'") and sourcePath.endswith("'"):
                    # its a literal in simple quotes!
                    literalsDict[targetPath] = sourcePath[1:-1]
                elif sourcePath in upgradeDict:
                    upgradeDict[sourcePath].append(targetPath)
                else:
                    upgradeDict[sourcePath] = [targetPath]
        # get dict of mappings with key=concept path, value=list of mapping objects
        mappings = oldCmf.removeMappings()
        newMappings = {}
        missingConcepts = {}
        for path, m in list(mappings.items()):
            if path in upgradeDict:
                log.debug("Move mappings from %s to %s" %(path, str(upgradeDict[path]) ))
                for targetPath in upgradeDict[path]:
                    if targetPath not in newMappings:
                        newMappings[targetPath] = []
                    newMappings[targetPath] += m[:]
            else:
                missingConcepts[path] = m
        # update newCmf
        newCmf.setMappings( newMappings )
        # add literals
        for path, literal in list(literalsDict.items()):
            mapObj = CMFMappingClass(items=[literal])
            newCmf.addMappingToConcept(path=path, mapObj=mapObj)
        # log missing concepts
        log.debug("The following missing concepts couldnt be included in the upgraded CMF:")
        for path, m in list(missingConcepts.items()):
            log.debug("Concept %s : %s" % (path, str(m) ))
        # return cmf and missing concepts mapping dict
        return newCmf, missingConcepts, upgradeTuple
